@@ -8,8 +8,10 @@ int main(void) {
 	struct passwd *pwinfo=NULL;
 	char *home=NULL, *home_tmp=NULL;
 	uid_t uid;
+	gid_t gid;
 
 	uid=getuid();
+	gid=getgid();
 
 	/* Don't chroot for uid==0 */
 	if (uid!=0) {
@@ -30,6 +32,12 @@ int main(void) {
 			return(3);
 		}
 	}
+
+	/* Set our GID and UID, GID first. */
+	if (setgid(gid)<0) return(5);
+	if (setuid(uid)<0) return(6);
+
+	/* Execute our shell. */
 	execl(SHELL, SHELL_SHORT, NULL);
 	return(4);
 }
